###########################################################################
###---Random forest model for detecting OTUs with high analyst signal---###

# Function rfmod requires dataframes: ss, wq, siteinfo, latlong, and taxonomist
# ss is a species (columns) by samples (rows) matrix
# wq is water quality data
# siteinfo is other environmental variables including physical habitat, ecoregion, etc.
# latlong is latitude and longitude of sites
# taxonomist is analyst identity of samples as binary factors
# All required dataframes have a common column of site IDs called "UID"
# Function returns a dataframe of OTUs ordered by highest analyst signal (mse.taxid.sc) and number of occurrence.


#################################
###---SET WORKING DIRECTORY---###
setwd("")


######################
###---LIBRARIES---####

library(foreach)
library(doParallel)
library(iterators)
library(parallel)
library(randomForest)

#####################
###---FUNCTIONS---###
source("functions/all_functions.R")


########################################################################
###---IMPORT WATER QUALITY, SITE, GEOGRPAPHIC, and TAXONOMIST DATA---### 
wq <- read.csv("read_only/1020_wq.csv", header=T)
siteinfo <- read.csv("read_only/0913_siteinfo.csv", header=T)
latlong <- read.csv("read_only/latlong.csv", header=T)
taxonomist <- read.csv("read_only/0913_taxonomist.csv", header=T)


########################################################################
###---IMPORT ss_mixed*_comp files DATA---###
ss_mixed1_comp <- read.csv("output/ss_mixed1_comp.csv", header=T)
ss_inter1_comp <- read.csv("output/ss_inter1_comp.csv", header=T)
ss_inter2_comp <- read.csv("output/ss_inter2_comp.csv", header=T)
ss_mixed2_comp <- read.csv("output/ss_mixed2_comp.csv", header=T)
ss_inter3_comp <- read.csv("output/ss_inter3_comp.csv", header=T)
ss_inter4_comp <- read.csv("output/ss_inter4_comp.csv", header=T)
ss_mixed3_comp <- read.csv("output/ss_mixed3_comp.csv", header=T)
ss_genus_comp <- read.csv("output/ss_genus_comp.csv", header=T)


#######################################################
###---RUN rfmod FUNCTION ON ss_mixed*_comp FILES---###
### Set seed for reproducible output (e.g., 111)
### Will take a long time depending on number of cpus and OTUs
### Print out: 
### number of OTUs with less than the minimum number of occurrences (default=50); 
### number of samples with less than 400 valves counted; 
### number of OTUs selected
### number of samples selected
### list of predictor variables included in random forest model
dfmse_mixed1 <- rfmod(ss_mixed1_comp, wq, siteinfo, latlong, taxonomist, 111)
dfmse_genus <- rfmod(ss_genus_comp, wq, siteinfo, latlong, taxonomist, 111)
dfmse_inter1 <- rfmod(ss_inter1_comp, wq, siteinfo, latlong, taxonomist, 111)
dfmse_inter2 <- rfmod(ss_inter2_comp, wq, siteinfo, latlong, taxonomist, 222)
dfmse_mixed2 <- rfmod(ss_mixed2_comp, wq, siteinfo, latlong, taxonomist, 222)
dfmse_inter3 <- rfmod(ss_inter3_comp, wq, siteinfo, latlong, taxonomist, 333)
dfmse_inter4 <- rfmod(ss_inter4_comp, wq, siteinfo, latlong, taxonomist, 444)
dfmse_mixed3 <- rfmod(ss_mixed3_comp, wq, siteinfo, latlong, taxonomist, 333)


#export random forest results
write.csv(dfmse_mixed1, "output/dfmse_mixed1.csv", row.names=FALSE)
write.csv(dfmse_genus, "output/dfmse_genus.csv", row.names=FALSE)
write.csv(dfmse_inter1, "output/dfmse_inter1.csv", row.names=FALSE)
write.csv(dfmse_inter2, "output/dfmse_inter2.csv", row.names=FALSE)
write.csv(dfmse_mixed2, "output/dfmse_mixed2.csv", row.names=FALSE)
write.csv(dfmse_inter3, "output/dfmse_inter3.csv", row.names=FALSE)
write.csv(dfmse_inter4, "output/dfmse_inter4.csv", row.names=FALSE)
write.csv(dfmse_mixed3, "output/dfmse_mixed3.csv", row.names=FALSE)
